#!/usr/bin/perl
use Getopt::Long;
use strict;

# Set default options
my ($help,$verbose,$test,$alpha,$html);
my $debug;
sub usage {
    return <<"EndUsage";
 Usage: $0 [name] [file]
 This interprets the file as votes in the Forge profiling
 Optional arguments:
 -h        Print this message
 -v        Verbose mode
 -test     Test mode, printing out all recognized game names
 -alpha    Print the games in alphabetical order
 -html     Output in HTML
EndUsage
}
GetOptions("h" => \$help, "help" => \$help, "v"=>\$verbose,
	   "alpha" => \$alpha, "html" => \$html, "debug" => \$debug, 
	   "test" => \$test);

if ($help) { print usage(); exit(); }

my $name = shift(@ARGV);
$name =~ m/200\d/ or die "Expect a year in the name";

die "Name and file arguments needed\n".usage() unless (scalar(@ARGV));

my ($username,$realname);
my %tracks = ("Last Played"=>{}, "Most Enjoyed"=>{},
	      "Want to Try"=>{});
my %gamehash;

my $nusers;
while (<>) {
    print "Parsing line $.\n" if ($debug);
    if (m/^"(\w[\w \.\']+[\w\.])"\s*(\((\w+)\))?/) {
	($username,$realname) = ($1,$3);
	$nusers++;
    } elsif (m/^(Last Played|Most Enjoyed|Want to Try):\s*(.*)/) {
	my ($category,$csv) = ($1,$2);
	my $href = $tracks{$category};

	# Ignore commas in parentheses - replace them with ";_;"
	while ($csv =~ m/\([^\)]*,/) {
	    $csv =~ s/(\([^\)]*),/$1<comma>/g;
	}

	my @games = split(/\s*,\s*/,$csv);
	die "Too many games in '$csv'" if (scalar(@games) > 3);
	foreach my $game (@games) {
	    my $note = $username;
	    die "No semicolons allowed" if ($game =~ m/ ; /);
	    if ($game =~ m/\{/) {
		print "Skipping '$game'\n" if ($verbose);
		next;
	    }
	    if ($game =~ s/(\(.*?\))//) {
		$note .= $1;
		$note =~ s/<comma>/,/g;
	    }
	    # remove leading and trailing spaces
	    $game =~ s/^\s+//; 
	    $game =~ s/\s+$//;
	    if ($game =~ m/D&D/) {
		if ($note =~ m/\(/) {
		    $note =~ s/\(/\($game - /;
		} else {
		    $note .= " ($game)";
		}
		$game = "D&D";
	    }
	    if ($href->{$game}) { $href->{$game}.=" ; "; }
	    $href->{$game}.=$note;
	    $gamehash{$game}++;
	    print "$game '$category' -> '$href->{$game}'\n" if ($verbose);
	}
    } elsif (not m/^\s*$/) {
	die "Bad line $.";
    }
}

if ($test) {
    print "Printing all game names in alphabetical order\n";
    foreach (sort keys %gamehash) {
	print "$_\n";
    }
    exit();
}

if ($html) {
    print <<"EndHeader";
<html> <head>
<title>$name Profiling</title>
</head>

<body bgcolor="ivory" marginwidth="24" leftmargin="24">
<h1>$name Profiling ($nusers responses)</h1>

<p>
<b>Categories:</b>
<ul>
  <li> <a href="#Last Played">
       <b>Last Played</b></a>
  <li> <a href="#Most Enjoyed">
       <b>Most Enjoyed</b></a>
  <li> <a href="#Want to Try">
       <b>Want to Try</b></a>
</ul>
</p>
EndHeader
}
foreach my $category (sort keys %tracks) {
    my $href = $tracks{$category};
    if ($html) {
	print "<hr><!------------------------------------------------>\n";
	print "<a name=\"$category\">\n";
	print "<h2>$category</h2></a>\n";
	print "<table border=\"1\">\n";
    } else {
	print "$category\n";
	print "==================\n";
    }
    foreach (keys %$href) {
	my $count = 1;
	while ($href->{$_} =~ m/ ; /g) { $count++; }
	$href->{$_} .= " [$count]";
	die "Code error" unless ($href->{$_});
    }
    my @by_popularity = sort {
	$href->{$a} =~ m/\[(\d+)\]/ or die "entry '$a'='$href->{$a}'";
	my $na = $1;
	$href->{$b} =~ m/\[(\d+)\]/ or die "entry '$b'='$href->{$b}'";
	my $nb = $1;
	($nb <=> $na) or ($a cmp $b);
    } keys %$href;
    my @alphabetical = sort keys %$href;

    my @gamelist = @by_popularity;
    if ($alpha) {
	@gamelist = @alphabetical;
    }
    foreach (@gamelist) {
	$href->{$_} =~ m/\[(\d+)\]/ or die "entry '$_'='$href->{$_}'";
	my $count = $1;
	if ($html) {
	    s/\'/&apos;/g;
	    printf "<tr>\n";
	    printf "  <th align=\"center\" width=\"30\">$count</th>\n";
	    printf "  <th align=\"left\">$_</th>\n";
	    printf "</tr>\n";
	    printf "<tr>\n";
	    s/&apos;/\'/g;
	    printf "  <td>&nbsp;</td><td><small>$href->{$_}</small></td>\n";
	    printf "</tr>\n";
	} else {
	    printf "%2d : $_\n     $href->{$_}\n",$count;
	}
    }
    if ($html) {
	print "</table>\n";
    }
}

if ($html) {
    print <<"EndFooter";
<hr><!------------------------------------------------>
<address>John H. Kim &lt;jhkim\@darkshire.org&gt;</address>
<!-- hhmts start -->
Last modified: Thu May  8 13:29:02 2003
<!-- hhmts end -->
</body> </html>
EndFooter
}
